//*=================================================================================*
// Inertia for Windows
// Example 4
//*=================================================================================*

//*=================================================================================*
// Headers
//*=================================================================================*
#include <windows.h>
#include "dibsect.h"            // Dude's DIB lib
#include "..\inertia\types.h"   // Inertia types
#include "..\inertia\inertia.h" // Inertia functions

//*=================================================================================*
// Global variables
//*=================================================================================*
ViewObject    View;           // a view for which to see everything
VectorObject  Obj1;           // an object we are going to look at
VectorObject  Obj2;           // another object we are going to look at
CDIBSection8 *VirtualScreen;  // 8-bit Device Independant Bitmap

// Caption for error dialog/message boxes
char *ErrorCaption = "Inertia Example 4 Error";

//*=================================================================================*
// Function headers
//*=================================================================================*
LRESULT CALLBACK EventHandler(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
int GeneralInit();
int CreateVirtualPage(HWND hwnd);
void CleanUp();

//*=================================================================================*
// MAIN
//*=================================================================================*
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR tmp3, int nCmdShow)
{
	MSG msg;
	HWND hwnd;
	WNDCLASS wndClass;

	if (!GeneralInit())  // start off by initializing some Inertia things
	{
		return(FALSE);
	}

	
	if (hPrevInstance == NULL)  // Create a new window class if necessairy
	{
		memset(&wndClass, 0, sizeof(wndClass));
		wndClass.style         = CS_HREDRAW | CS_VREDRAW;
		wndClass.lpfnWndProc   = EventHandler;        // use our event handler
		wndClass.hInstance     = hInstance;
		wndClass.hCursor       = LoadCursor(NULL, IDC_ARROW);
		wndClass.lpszClassName = "Inertia_Window_Class";

		if (!RegisterClass(&wndClass))
			return FALSE;
	}

	// Create/Open the window
	hwnd = CreateWindow("Inertia_Window_Class", "Inertia Example 4",
						WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, 0, 640, 480, NULL, NULL,
						hInstance, NULL);


	// Create the virtual page for us to render on
	if (!CreateVirtualPage(hwnd))
	{
		CleanUp();
		return(FALSE);
	}

	ShowWindow(hwnd, nCmdShow);

	UpdateWindow(hwnd);

	while (TRUE)  // main execution loop
	{
		SetDeltaRotation(&Obj1, 0.01f, 0.02f, 0.03f);  // rotate the object  
		SetDeltaRotation(&Obj2, 0.01f, 0.02f, 0.03f);  // rotate the object  
		AddToRenderList(&View, &Obj1);
		AddToRenderList(&View, &Obj2);
		Render((char *)VirtualScreen->GetBuffer(), VirtualScreen->GetPitch(), 480);

		VirtualScreen->Update();  // Page flip

		// Check for any windows messages
        if (PeekMessage (&msg, NULL, 0, 0, PM_REMOVE))
		{
			if ((msg.message == WM_QUIT) || (msg.message == WM_DESTROY))
			{
			  CleanUp();
			  return (msg.wParam);
			}

          TranslateMessage (&msg);
          DispatchMessage (&msg);
		}

		// clear the virtual page
		memset(VirtualScreen->GetBuffer(), 0, VirtualScreen->GetPitch() * VirtualScreen->GetHeight());
	}

	return(msg.wParam);
}

//*=================================================================================*
// EventHandler - Handles windows specific events
//*=================================================================================*
LRESULT CALLBACK EventHandler(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_DESTROY: CleanUp(); PostQuitMessage(0); exit(0); break;
	case WM_SIZE:   // The window has been re-sized
		delete VirtualScreen;
		if (!CreateVirtualPage(hwnd))
		{
			CleanUp();
			PostQuitMessage(0);
			exit(0);
		}

		SetViewPort(&View, 0, 0, (float)VirtualScreen->GetWidth() - 1, (float)VirtualScreen->GetHeight() - 1, 1);
		return(0);
		break;
	}

	return (DefWindowProc(hwnd, uMsg, wParam, lParam));
}

//*=================================================================================*
// CleanUp - cleans up and dynamic memory used
//*=================================================================================*
void CleanUp()
{
	delete VirtualScreen;

	FreeIntensityTable();
	FreeTransparencyTable();
	FreeVectorObject(&Obj1);
	FreeVectorObject(&Obj2);
	FreeTexture(0);

	ShutDownInertia();
}

//*=================================================================================*
// CreateVirtualPage - creates a virtual page with palette
//*=================================================================================*
int CreateVirtualPage(HWND hwnd)
{
	unsigned char TempPalette[256*3];
	int  i;

	for (i = 0; i < 256; i++)
	{
		TempPalette[i*3+0] = GlobalPalette[i].r << 2;
		TempPalette[i*3+1] = GlobalPalette[i].g << 2;
		TempPalette[i*3+2] = GlobalPalette[i].b << 2;
	}
		
	// create an 8-bit device independant bitmap.
	// The first parameter is the HWND of the window we are using
	// The second parameter is the rectangular drawing area. If NULL, the size of the windows is used
	// The third parameter is the palette we want to use for the window

	VirtualScreen = new CDIBSection8(hwnd, NULL, TempPalette);

	if (VirtualScreen == NULL)
	{
		MessageBox(NULL, "Error! Virtual page could not be allocated.", ErrorCaption, MB_OK);
		return(0);
	}

	return (1);
}

//*=================================================================================*
// GeneralInit - Performs Inertia initialization
//*=================================================================================*
int GeneralInit()
{
	InitInertia();   // initialize Inertia
	
	// Load the palette data
	if (LoadPCXTexture(0, "COPPRENV.PCX", 0) != I_OK)
	{
		MessageBox(NULL, "Error! Texture data could not be loaded.", ErrorCaption, MB_OK);
		return (0);
	}

	GetTexturePalette(0);   // grab the pallete from the texture map
    
	// load the intensity table from disk
	if (LoadIntensityTable("COPPRENV.IT", 0) != I_OK) 
	{	
     	MessageBox(NULL, "Error! Intensity table data file could not be loaded.", ErrorCaption, MB_OK);
		return (0);
	}

	if (LoadTransparencyTable("COPPRENV.TT", 0) != I_OK) 
	{	
     	MessageBox(NULL, "Error! Transparency table data file could not be loaded.", ErrorCaption, MB_OK);
		return (0);
	}

	// load the 3d model data from disk		
	if (LoadGVO(&Obj1, "TORUSA.GVO", 0) != I_OK)        
	{
		MessageBox(NULL, "Error! TorusA.gvo file could not be loaded.", ErrorCaption, MB_OK);     
		return(0);
	}

	// load the 3d model data from disk		
	if (LoadGVO(&Obj2, "TORUSB.GVO", 0) != I_OK)        
	{
		MessageBox(NULL, "Error! TorusB.gvo file could not be loaded.", ErrorCaption, MB_OK);     
		return(0);
	}

	InitView(&View);

	SetViewPort(&View, 0, 0, 640-1, 480-1, 1);

	SetAbsoluteLocation(&Obj1, 0, 0, Obj1.Radius * 2); 
	SetAbsoluteLocation(&Obj2, 0, 0, Obj2.Radius * 2); 

	PushLightSource(0, 0, -1);  

	SetRenderMask(&Obj1, TRANSPARENT | SMOOTH | GOURAUD);   // set to transparent gouraud
	SetCull(&Obj1, 0);			// turn off culling
	SetBaseColor(&Obj1, 36);

	SetRenderMask(&Obj2, TEXTURE | GOURAUD); // gouraud texture mapping
	SetObjectTexture(&Obj2, 0);

	return(1);  // if we make it this far, all is good, so return success
}